﻿Shader "NPR/ScreenSpaceStrokesShader"
{
	Properties
	{
		[NoScaleOffset] _MainTex ("Texture", 2D) = "black" {}		
	}
	SubShader
	{
		Tags { "Queue" = "Overlay" }
		
		GrabPass
		{
			"_BackgroundTexture"
		}

		Pass
		{
			Cull Off
			ZWrite Off
			ZTest Off
			Blend SrcAlpha OneMinusSrcAlpha

			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
				float4 color : TEXCOORD1;
			};

			
			float4 strokeSize;
			sampler2D _BackgroundTexture;
			v2f vert (appdata v)
			{				
				v2f o;
				float3 rgb2lum = float3(0.2126, 0.7152, 0.0722);
				float2 screenPos = ComputeGrabScreenPos(float4(v.vertex.xy,0,1)).xy;
				
				float2 invPixelSize = float2(2.0 / _ScreenParams.x, 2.0 / _ScreenParams.y);
				float dLdx = -1 * dot(tex2Dlod(_BackgroundTexture, float4(screenPos + invPixelSize * float2(-1, 1), 0, 0)).rgb, rgb2lum) + 
							 -2 * dot(tex2Dlod(_BackgroundTexture, float4(screenPos + invPixelSize * float2(-1, 0), 0, 0)).rgb, rgb2lum) +
				             -1 * dot(tex2Dlod(_BackgroundTexture, float4(screenPos + invPixelSize * float2(-1,-1), 0, 0)).rgb, rgb2lum) +
					          1 * dot(tex2Dlod(_BackgroundTexture, float4(screenPos + invPixelSize * float2( 1, 1), 0, 0)).rgb, rgb2lum) +
							  2 * dot(tex2Dlod(_BackgroundTexture, float4(screenPos + invPixelSize * float2( 1, 0), 0, 0)).rgb, rgb2lum) +
							  1 * dot(tex2Dlod(_BackgroundTexture, float4(screenPos + invPixelSize * float2( 1,-1), 0, 0)).rgb, rgb2lum);
				float dLdy = -1 * dot(tex2Dlod(_BackgroundTexture, float4(screenPos + invPixelSize * float2(-1, 1), 0, 0)).rgb, rgb2lum) +
							 -2 * dot(tex2Dlod(_BackgroundTexture, float4(screenPos + invPixelSize * float2( 0, 1), 0, 0)).rgb, rgb2lum) +
							 -1 * dot(tex2Dlod(_BackgroundTexture, float4(screenPos + invPixelSize * float2( 1, 1), 0, 0)).rgb, rgb2lum) +
							  1 * dot(tex2Dlod(_BackgroundTexture, float4(screenPos + invPixelSize * float2(-1,-1), 0, 0)).rgb, rgb2lum) +
					          2 * dot(tex2Dlod(_BackgroundTexture, float4(screenPos + invPixelSize * float2( 0,-1), 0, 0)).rgb, rgb2lum) +
					          1 * dot(tex2Dlod(_BackgroundTexture, float4(screenPos + invPixelSize * float2( 1,-1), 0, 0)).rgb, rgb2lum);
				
				float2 dir = normalize(float2(1, 1));
				float2 dirPerp = normalize(float2(-1, 1));				
				if (dLdx != 0 || dLdy != 0)
				{
					dir = normalize(float2(dLdx, dLdy));
					dirPerp = float2(-dir.y, dir.x);
				}				
				o.vertex = float4(v.vertex.xy + strokeSize.x * v.uv.x * dirPerp + 
												strokeSize.y * v.uv.y * dir, 0, 1);
				o.uv = v.uv;
				
				float4 screenColor = tex2Dlod(_BackgroundTexture, float4(screenPos, 0, 0));
				o.color = screenColor;
				float Lum = dot(screenColor.rgb, rgb2lum);
				float priority = v.vertex.z;
				if (priority < Lum)
					o.vertex.z = -2;
				
				return o;
			}
			
			sampler2D _MainTex;
			float4 strokeColor;
			float4 frag (v2f i) : SV_Target
			{
				float strokeAlpha = 1.0 - tex2D(_MainTex, i.uv).r;
			    //return float4(i.color.rgb, strokeColor.a * strokeAlpha);
				return float4(strokeColor.rgb, strokeColor.a * strokeAlpha);
			}
			ENDCG
		}
	}
}
