﻿Shader "NPR/TexturedStrokes"
{
	Properties
	{
		
	}
	SubShader
	{
		Tags { "RenderType"="Opaque" }
		LOD 100

		Pass
		{
			Tags{ "LightMode" = "ForwardAdd" }
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag

			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float3 normal : NORMAL;
				float2 texcoord : TEXCOORD;
			};

			struct v2f
			{
				float3 normal : NORMAL;
				float2 texcoord : TEXCOORD0;
				float3 lightDir : TEXCOORD1;
				float4 vertex : SV_POSITION;
			};

			v2f vert(appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.texcoord = v.texcoord;
				o.normal = UnityObjectToWorldNormal(v.normal);
				float4 worldPos = mul(unity_ObjectToWorld, v.vertex);
				o.lightDir = normalize(_WorldSpaceLightPos0.xyz -
					worldPos * _WorldSpaceLightPos0.w);
				return o;
			}

			uniform sampler3D _StrokeMap;
			uniform float2 textureScale;
			float4 frag(v2f i) : SV_Target
			{
				float3 N = normalize(i.normal);
				float3 L = normalize(i.lightDir);
				float tone = max(0,dot(N,L));
				float slice = floor(tone * 4);
				float3 stroke = tex3D(_StrokeMap, float3(i.texcoord * textureScale, tone)).rgb;
				return float4(stroke, 1);
			}


			ENDCG
		}
	}
}
