﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[ExecuteInEditMode]
[RequireComponent(typeof(Camera))]
public class ColorTransferEffect : PostEffectsBase {

    public Texture exemplarTexture;

	void Start () {
        shader = Shader.Find("NPR/ColorTransferShader");
        CreateMaterial();
    }

    // meanTex is assumed to have all mip-map levels generated
    // The finest level contains the original pixels values, the coarsest level is the mean pixel value.
    RenderTexture CreateVarianceTexture(Texture meanTex)
    {
        RenderTexture varianceTex = RenderTexture.GetTemporary(meanTex.width, meanTex.height, 0, RenderTextureFormat.DefaultHDR);
        varianceTex.useMipMap = true;
        int maxLevel = (int)Mathf.Log(Mathf.Max(meanTex.width, meanTex.height), 2); // maximum mip-map level (1 pixel)
        material.SetInt("singlePixelLevel", maxLevel);
        Graphics.Blit(meanTex, varianceTex, material, 0); // pass no. 0: variance computation shader
        return varianceTex;
    }

    void OnRenderImage(RenderTexture source, RenderTexture destination) {
        if (CreateMaterial() && exemplarTexture != null)
        {
            // compute mean for the exemplar (target) texture
            RenderTexture meanTexExemplar = RenderTexture.GetTemporary(exemplarTexture.width, exemplarTexture.height);
            meanTexExemplar.useMipMap = true;
            Graphics.Blit(exemplarTexture, meanTexExemplar);
            // compute variance for the exemplar (target) texture
            RenderTexture varianceTexExemplar = CreateVarianceTexture(meanTexExemplar);

            // compute mean for source texture
            RenderTexture meanTexSource = RenderTexture.GetTemporary(source.width, source.height);
            meanTexSource.useMipMap = true;
            Graphics.Blit(source, meanTexSource);
            // compute variance for source texture
            RenderTexture varianceTexSource = CreateVarianceTexture(meanTexSource);

            int maxLevelSource = (int)Mathf.Log(Mathf.Max(meanTexSource.width, meanTexSource.height), 2); // maximum mip-map level (1 pixel)
            material.SetInt("singlePixelLevelSource", maxLevelSource);
            int maxLevelExemplar = (int)Mathf.Log(Mathf.Max(meanTexExemplar.width, meanTexExemplar.height), 2); // maximum mip-map level (1 pixel)
            material.SetInt("singlePixelLevelExemplar", maxLevelExemplar);
            material.SetTexture("meanTexSource", meanTexSource);
            material.SetTexture("meanTexExemplar", meanTexExemplar);
            material.SetTexture("varianceTexSource", varianceTexSource);
            material.SetTexture("varianceTexExemplar", varianceTexExemplar);
            Graphics.Blit(source, destination, material, 1);

            RenderTexture.ReleaseTemporary(meanTexSource);
            RenderTexture.ReleaseTemporary(meanTexExemplar);
            RenderTexture.ReleaseTemporary(varianceTexSource);
            RenderTexture.ReleaseTemporary(varianceTexExemplar);
        }
        else
        {
            Graphics.Blit(source, destination);
        }
    }
}
