﻿Shader "NPR/ColorTransferShader"
{
	Properties
	{
		_MainTex ("Texture", 2D) = "white" {}
	}
	SubShader
	{
		// No culling or depth
		Cull Off ZWrite Off ZTest Always

		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert (appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}
			
			sampler2D _MainTex;
			int singlePixelLevel;

			float4 frag (v2f i) : SV_Target
			{
				float4 pixel = tex2D(_MainTex, i.uv); // x
				float4 mean = tex2Dlod(_MainTex, float4 (0.5,0.5 , 0, singlePixelLevel)); // E[x]

				float4 diff = pixel - mean; // x - E[x]
				return diff*diff; // (x - E[x])^2 -> variance = E[(x - E[x])^2]
			}
			ENDCG
		}

		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert (appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}
			
			sampler2D _MainTex;
			sampler2D meanTexSource;
			sampler2D meanTexExemplar;
			sampler2D varianceTexSource;
			sampler2D varianceTexExemplar;
			int singlePixelLevelSource;
			int singlePixelLevelExemplar;

			float4 frag (v2f i) : SV_Target
			{
				float4 pixel = tex2D(_MainTex, i.uv);
				float4 sourceMean = tex2Dlod(meanTexSource, float4 (0.5,0.5 , 0, singlePixelLevelSource));
				float4 targetMean = tex2Dlod(meanTexExemplar, float4 (0.5, 0.5, 0, singlePixelLevelExemplar));
				float4 sourceVariance = tex2Dlod(varianceTexSource, float4 (0.5, 0.5, 0, singlePixelLevelSource));
				float4 targetVariance = tex2Dlod(varianceTexExemplar, float4 (0.5, 0.5, 0, singlePixelLevelExemplar));

				pixel = pixel - sourceMean;
				float4 varianceScale = targetVariance / sourceVariance;
				//pixel *= sqrt(varianceScale); // sometimes it looks better if we scale the standard deviation instead of the variance
				pixel *= varianceScale;
				pixel += targetMean;

				return pixel;
			}
			ENDCG
		}
	}
}
